"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const solc = require("solc");
const fs = require("fs");
const fsx = require("fs-extra");
const pth = require("path");
const glob = require("fast-glob");
class Compiler {
    constructor(cwd) {
        this.contracts = [];
        this.cwd = cwd || process.cwd();
    }
    require(...patterns) {
        const files = this.getFiles(patterns);
        const output = this.compileFiles(files);
        for (const key in output.contracts) {
            const data = output.contracts[key];
            const [sourcePath, contractName] = key.split(':');
            this.contracts.push(Object.assign({}, data, { contractName, abi: JSON.parse(data.interface), sourcePath: pth.resolve(this.normalizePath(sourcePath)) }));
        }
        return this;
    }
    serialize() {
        return JSON.parse(JSON.stringify(this.contracts));
    }
    clear() {
        this.contracts = [];
        return this;
    }
    save(dist) {
        const modulePath = pth.join(this.cwd, 'node_module');
        const targetPath = pth.resolve(this.cwd, dist);
        fsx.ensureDirSync(targetPath);
        this.contracts.forEach((contract) => {
            const isModule = contract.sourcePath.indexOf(modulePath) === 0;
            if (!isModule) {
                const path = pth.join(targetPath, `${contract.contractName}.json`);
                const data = JSON.stringify(contract, null, 2);
                fs.writeFileSync(path, data);
            }
        });
    }
    getFiles(patterns) {
        return glob.sync(patterns, { cwd: this.cwd });
    }
    compileFiles(files) {
        const sources = {};
        files.forEach((f) => {
            sources[f] = fs.readFileSync(f).toString();
        });
        return solc.compile({ sources }, 1, (path) => {
            return {
                contents: fs.readFileSync(this.normalizePath(path)).toString(),
            };
        });
    }
    normalizePath(path) {
        return path.indexOf('@') === 0 ? `node_modules/${path}` : path;
    }
}
exports.Compiler = Compiler;
//# sourceMappingURL=compiler.js.map